use super::EntityId;
use crate::protocol::mctypes::{Position, VarInt};

#[derive(Debug, Copy, Clone, PartialEq)]
pub struct Particle {
    kind: ParticleKind,
}

#[repr(u8)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub enum ParticleKind {
    AmbientEntityEffect = 0,
    AngryVillager = 1,
    // TODO: Add the block state
    Block(()) = 2,
    // TODO: Add the block state
    BlockMarker(()) = 3,
    Bubble = 4,
    Cloud = 5,
    Crit = 6,
    DamageIndicator = 7,
    DragonBreath = 8,
    DrippingLava = 9,
    FallingLava = 10,
    LandingLava = 11,
    DrippingWater = 12,
    FallingWater = 13,
    Dust {
        rgb: [f32; 3],
        scale: f32,
    } = 14,
    DustColorTransition {
        from_rgb: [f32; 3],
        scale: f32,
        to_rgb: [f32; 3],
    } = 15,
    Effect = 16,
    ElderGuardian = 17,
    EnchantedHit = 18,
    Enchant = 19,
    EndRod = 20,
    EntityEffect = 21,
    ExplosionEmitter = 22,
    Explosion = 23,
    SonicBoom = 24,
    // TODO: Add the block state
    FallingDust(()) = 25,
    Firework = 26,
    Fishing = 27,
    Flame = 28,
    DrippingCherryLeaves = 29,
    FallingCherryLeaves = 30,
    LandingCherryLeaves = 31,
    SkulkSoul = 32,
    SkulkCharge {
        roll: f32,
    } = 33,
    SkulkChargePop = 34,
    SoulFireFlame = 35,
    Soul = 36,
    Flash = 37,
    HappyVillager = 38,
    Composter = 39,
    Heart = 40,
    InstantEffect = 41,
    // TODO: Add the slot
    Item(()) = 42,
    Vibration {
        source: VibrationParticleSource,
        travel_duration_ticks: VarInt,
    } = 43,
    ItemSlime = 44,
    ItemSnowball = 45,
    LargeSmoke = 46,
    Lava = 47,
    Mycelium = 48,
    Note = 49,
    Poof = 50,
    Portal = 51,
    Rain = 52,
    Smoke = 53,
    Sneeze = 54,
    Spit = 55,
    SquidInk = 56,
    SweepAttack = 57,
    TotemOfUndying = 58,
    Underwater = 59,
    Splash = 60,
    Witch = 61,
    BubblePop = 62,
    CurrentDown = 63,
    BubbleColumnUp = 64,
    Nautilus = 65,
    Dolphin = 66,
    CampfireCosySmoke = 67,
    CampfireSignalSmoke = 68,
    DrippingHoney = 69,
    FallingHoney = 70,
    LandingHoney = 71,
    FallingNectar = 72,
    FallingSporeBlossom = 73,
    Ash = 74,
    CrimsonSpore = 75,
    WarpedSpore = 76,
    SporeBlossomAir = 77,
    DrippingObsidianTear = 78,
    FallingObsidianTear = 79,
    LandingObsidianTear = 80,
    ReversePortal = 81,
    WhiteAsh = 82,
    SmallFlame = 83,
    Snowflake = 84,
    DrippingDripstoneLava = 85,
    FallingDripstoneLava = 86,
    DrippingDripstoneWater = 87,
    FallingDripstoneWater = 88,
    GlowSquidInk = 89,
    Glow = 90,
    WaxOn = 91,
    WaxOff = 92,
    ElectricSpark = 93,
    Scrape = 94,
    Shriek {
        delay_ticks: VarInt,
    } = 95,
}

#[derive(Debug, Copy, Clone, PartialEq)]
pub enum VibrationParticleSource {
    Block(Position),
    Entity { id: EntityId, eye_height: f32 },
}
